%This Matlab code performs the following

% 1a) Loads the file Spikes.mat which contains 2000 trials of LIF spikes,
% each of these spike trains has a duration of 1100 ms. These spikes were generated 
% using an LIF model driven by bimodal inputs.

% p.s. 1b) if you want to see the functions that generated these spikes or if 
%         you want to use a different set of spikes and load those, please check the file SpikegenerationLIF.m.


% 2) Calculationf of the Fourier coefficients
% 3) Reverse transformation of Fourier coefficients to obtain spikes. 
%    Here we use the first trial of spikes.
%    As you can see spikes are discrete events in time but they can nevertheless
%    be obtained from a Gaussian distributed Fourier coefficients.
% 4) Generation of a figure showing the original and reconstructed spike times.
% 5) Evaluation of the distribution of these Fourier coefficients for a given frequency, 
%    to verify that the distribution is Gaussian
% 6) Generation of a figure showing the Gaussian distribution of Fourier Coefficients. 
%    Note that this distribution is obtained from 2000 trials where each trial is 1100 sec long.


% Output of the script: two figures
% Figure 1: original and reconstructed spike times
% Figure 2: Gaussian distribution of Fourier coefficients

%% 1a. Loading of the spike train

load('Spike.mat');

% Given spike train:
t_s_n=t_s_u{1,1};


%%  timestep size of the voltage process

dt = 0.1;
% recording length
T_sequence    = 1100; % (ms)  
steps = ceil(T_sequence/dt);
% Number of Trials
Ntr=2000;

%% 1b: Alternative: Here you can generate the spikes with the script 'SpikegenerationLIF'
%% Spike train generation with LIF with reset and bimodal input distribution per time step 
%  [t_s_u,t_s_r] = SpikegenerationLIF(Ntr,steps, dt);
% 
% % Given spike train:
% t_s_n=t_s_u{1};
% 
% 
%    save('Spike.mat','t_s_u','t_s_r');
%% 2. Generation of the Fourier cofficients 
% Generation of the real Fourier cofficients from the spike train in the arry t_s_n

nmax=0.5*(T_sequence/dt);
n=(1:nmax);
p =T_sequence; %  T_sequence is defined as periode 
w=2*pi/(T_sequence);

coef_cos_test = zeros(1,length(n));
coef_sin_test = zeros(1,length(n));


    
 for f = 1:length(n)
     
coef_cos_test(1,f) = 2/T_sequence*sum(cos(w*f*t_s_n),2);
coef_sin_test(1,f) = 2/T_sequence*sum(sin(w*f*t_s_n),2);


 end 

%% 3) Reverse transformation of Fourier coefficients to obtain spikes. 

% Fourier series 
coef_zero=(2/T_sequence)*length(t_s_n);
t2=(0:dt:T_sequence);

Spiketrain_test = zeros(1,length(t2));
for j=1:length(t2)
 
Spiketrain_test(j)=coef_zero/2+ sum(coef_cos_test.*cos(n*w*t2(j)))+sum(coef_sin_test.*sin(n*w*t2(j)));

end

%% 4. Comparison of the original spike train and its reconstruction from Fourier coefficients

fig1=figure(1);
figure(1);
set(fig1, 'Position', [100, 100, 1000, 200])
set(gca,'fontsize', 12)

plot(t2,Spiketrain_test)
hold on
 for i = 1:length(t_s_n)
     line([t_s_n(i) t_s_n(i)],[0 1/dt],'LineWidth',dt,'Color','black');
 end
  
title('Reconstruction of spike times from Gaussian Fourier coefficients, original (black), reconstruction (blue)')
xlabel('Time [ms]')
ylabel('Spike trains')

 Intetest=dt*trapz(Spiketrain_test);  % equal to number of spikes
 
 display(strcat('Comparing spike numbers in the original and the reconstructed spike train'));
 display(strcat(['#Spikes in the reconstructed spike train ', num2str(Intetest)]));
 display(strcat(['#Spikes in the original spike train ', num2str(length(t_s_n))]));



%% 5. Distribution of Fourier coefficients
%% Properties

plot_freq   = 11; %Hz
%% Specify Frequencies

freqs_deltaf = 1/T_sequence*1000;
freq         = ceil(plot_freq/freqs_deltaf)*freqs_deltaf;



%% Fourier coefficients of spike trains with varying stimulus
%% Calculate sine and cosine coefficients


t_s = t_s_u';



coef_complex_u = zeros(Ntr,1);

coef_cos_u = zeros(Ntr,1);
coef_sin_u = zeros(Ntr,1);

for i = 1 : Ntr 


   coef_cos_u(i,1)     = 1000*2/T_sequence*sum(cos(2*pi*freq*t_s{1,i}/1000),2)';
   coef_sin_u(i,1)     = 1000*2/T_sequence*sum(sin(2*pi*freq*t_s{1,i}/1000),2)';

end



%% Further calculation (variances and means)


var_cos_u = var(coef_cos_u(:,1));
var_sin_u = var(coef_sin_u(:,1));


mean_cos_u = mean(coef_cos_u(:,1));
mean_sin_u = mean(coef_sin_u(:,1));

%% Calculating the distribution of the Fourier coefficients (histogram)

bins_min = -20;
bins_max =  20;
bins_count = 30;
bins_cos_u = linspace(bins_min,bins_max,bins_count);
bins_delta_cos_u = bins_cos_u(2)-bins_cos_u(1);

coef_count = size(coef_cos_u,1);
hist_cos_u = histc(coef_cos_u,bins_cos_u)/coef_count/bins_delta_cos_u;


norm_x_count = 10000;
norm_x = linspace(bins_cos_u(1),bins_cos_u(end),norm_x_count);

norm_dis_cos_u = 1/sqrt(2*pi*var_cos_u)*exp(-(norm_x-mean_cos_u).^2/2/var_cos_u);


delta_pres_cos = bins_delta_cos_u/4;

%%  6. Figure generation of the Gaussian distribution of the Fourier
% coefficients

fig2 = figure;
set(gca,'fontsize', 12)
bar(bins_cos_u,hist_cos_u,0.5,'EdgeColor','none')
hold on
plot(norm_x,norm_dis_cos_u,'k','LineWidth',3)


xlim([-20 20]);

title('Fourier coefficients in spiking response')
xlabel('Re of c_R')
ylabel('Probability density')



